@@ -1,30 +1,31 @@
 import json
 import logging
 
+import reversion
 from django import forms
 from django.conf import settings
-from django.conf.urls import url
 from django.contrib import admin, messages
 from django.contrib.admin import helpers
+from django.contrib.admin.models import ADDITION, LogEntry
+from django.contrib.contenttypes.models import ContentType
 from django.core.exceptions import (
     FieldDoesNotExist,
     ObjectDoesNotExist,
     ValidationError,
 )
-from django.http import Http404, HttpResponse
+from django.http import Http404, HttpResponse, JsonResponse
 from django.shortcuts import get_object_or_404
 from django.template.loader import get_template
 from django.template.response import TemplateResponse
-from django.urls import reverse
-from django.utils.translation import ugettext_lazy as _
+from django.urls import path, re_path, reverse
+from django.utils.translation import gettext_lazy as _
 from flat_json_widget.widgets import FlatJsonWidget
 from swapper import load_model
 
 from openwisp_controller.config.views import (
-    get_default_templates,
+    get_relevant_templates,
     get_template_default_values,
 )
-from openwisp_users.models import Organization
 from openwisp_users.multitenancy import (
     MultitenantOrgFilter,
     MultitenantRelatedOrgFilter,
@@ -38,15 +39,18 @@ from openwisp_utils.admin import (
 from ..admin import MultitenantAdminMixin
 from ..pki.base import PkiReversionTemplatesMixin
 from . import settings as app_settings
+from .base.vpn import AbstractVpn
 from .utils import send_file
-from .widgets import JsonSchemaWidget
+from .widgets import DeviceGroupJsonSchemaWidget, JsonSchemaWidget
 
 logger = logging.getLogger(__name__)
 prefix = 'config/'
 Config = load_model('config', 'Config')
 Device = load_model('config', 'Device')
+DeviceGroup = load_model('config', 'DeviceGroup')
 Template = load_model('config', 'Template')
 Vpn = load_model('config', 'Vpn')
+Organization = load_model('openwisp_users', 'Organization')
 OrganizationConfigSettings = load_model('config', 'OrganizationConfigSettings')
 
 if 'reversion' in settings.INSTALLED_APPS:
@@ -99,6 +103,10 @@ class BaseConfigAdmin(BaseAdmin):
                 }
             ]
         }
+        # do not pass CONFIG_BACKEND_FIELD_SHOWN in VpnAdmin
+        # since we don't need to hide the VPN backend there
+        if not issubclass(self.model, AbstractVpn):
+            ctx['CONFIG_BACKEND_FIELD_SHOWN'] = app_settings.CONFIG_BACKEND_FIELD_SHOWN
         if pk:
             ctx['download_url'] = reverse('{0}_download'.format(prefix), args=[pk])
             try:
@@ -130,17 +138,17 @@ class BaseConfigAdmin(BaseAdmin):
         options = getattr(self.model, '_meta')
         url_prefix = '{0}_{1}'.format(options.app_label, options.model_name)
         return [
-            url(
+            re_path(
                 r'^download/(?P<pk>[^/]+)/$',
                 self.admin_site.admin_view(self.download_view),
                 name='{0}_download'.format(url_prefix),
             ),
-            url(
-                r'^preview/$',
+            path(
+                'preview/',
                 self.admin_site.admin_view(self.preview_view),
                 name='{0}_preview'.format(url_prefix),
             ),
-            url(
+            re_path(
                 r'^(?P<pk>[^/]+)/context\.json$',
                 self.admin_site.admin_view(self.context_view),
                 name='{0}_context'.format(url_prefix),
@@ -155,10 +163,11 @@ class BaseConfigAdmin(BaseAdmin):
 
     def _get_preview_instance(self, request):
         """
-        returns a temporary preview instance used for preview
+        returns a temporary instance used for preview
         """
         kwargs = {}
         config_model = self._get_config_model()
+        instance = config_model()
         for key, value in request.POST.items():
             # skip keys that are not model fields
             try:
@@ -183,9 +192,17 @@ class BaseConfigAdmin(BaseAdmin):
         # default context to None to avoid exception
         if 'context' in kwargs:
             kwargs['context'] = kwargs['context'] or None
-        # this object is instanciated only to generate the preview
-        # it won't be saved to the database
-        instance = config_model(**kwargs)
+        try:
+            instance = config_model.objects.get(pk=request.POST['id'])
+            for key, value in kwargs.items():
+                setattr(instance, key, value)
+        except (KeyError, ValidationError, config_model.DoesNotExist):
+            # this object is instanciated only to generate the preview
+            # it won't be saved to the database
+            instance = config_model(**kwargs)
+        # turn off special name validation
+        # (see ``ShareableOrgMixinUniqueName``)
+        instance._validate_name = False
         instance.full_clean(exclude=['device'], validate_unique=False)
         return instance
 
@@ -213,18 +230,24 @@ class BaseConfigAdmin(BaseAdmin):
         template_ids = request.POST.get('templates')
         if template_ids:
             template_model = config_model.get_template_model()
+            template_ids = template_ids.split(',')
             try:
-                templates = template_model.objects.filter(
-                    pk__in=template_ids.split(',')
-                )
+                templates = template_model.objects.filter(pk__in=template_ids)
                 templates = list(templates)  # evaluating queryset performs query
+                # ensure the order of templates is maintained
+                templates.sort(
+                    key=lambda template: template_ids.index(str(template.id))
+                )
             except ValidationError as e:
                 logger.exception(error_msg, extra={'request': request})
                 return HttpResponse(str(e), status=400)
         else:
             templates = None
         if not error:
-            backend = instance.get_backend_instance(template_instances=templates)
+            context = instance.get_context()
+            backend = instance.get_backend_instance(
+                template_instances=templates, context=context
+            )
             try:
                 instance.clean_netjsonconfig_backend(backend)
                 output = backend.render()
@@ -324,21 +347,27 @@ class ConfigForm(AlwaysHasChangedMixin, BaseForm):
                 reverse=False,
                 model=config.templates.model,
                 pk_set=templates,
+                # The template validation retrieves the device object
+                # from the database. Even if the organization of the device
+                # is changed by the user, the validation uses the old
+                # organization of the device because the device is not
+                # saved yet. The raw POST data is passed here so that
+                # validation can be performed using up to date data of
+                # the device object.
+                raw_data=self.data,
             )
         return templates
 
     class Meta(BaseForm.Meta):
         model = Config
         widgets = {'config': JsonSchemaWidget, 'context': FlatJsonWidget}
-        labels = {
-            'context': _('Configuration Variables'),
-        }
+        labels = {'context': _('Configuration Variables')}
         help_texts = {
             'context': _(
                 "In this section it's possible to override the default values of "
                 "variables defined in templates. If you're not using configuration "
                 "variables you can safely ignore this section."
-            ),
+            )
         }
 
 
@@ -374,9 +403,11 @@ class ConfigInline(
 
 
 class DeviceAdmin(MultitenantAdminMixin, BaseConfigAdmin, UUIDAdmin):
+    recover_form_template = 'admin/config/device_recover_form.html'
     list_display = [
         'name',
         'backend',
+        'group',
         'config_status',
         'mac_address',
         'ip',
@@ -384,19 +415,31 @@ class DeviceAdmin(MultitenantAdminMixin, BaseConfigAdmin, UUIDAdmin):
         'modified',
     ]
     list_filter = [
+        'config__status',
         ('organization', MultitenantOrgFilter),
         ('config__templates', MultitenantRelatedOrgFilter),
-        'config__status',
+        ('group', MultitenantRelatedOrgFilter),
         'created',
     ]
-    search_fields = ['id', 'name', 'mac_address', 'key', 'model', 'os', 'system']
+    search_fields = [
+        'id',
+        'name',
+        'mac_address',
+        'key',
+        'model',
+        'os',
+        'system',
+        'devicelocation__location__address',
+    ]
     readonly_fields = ['last_ip', 'management_ip', 'uuid']
+    autocomplete_fields = ['group']
     fields = [
         'name',
         'organization',
         'mac_address',
         'uuid',
         'key',
+        'group',
         'last_ip',
         'management_ip',
         'model',
@@ -407,11 +450,12 @@ class DeviceAdmin(MultitenantAdminMixin, BaseConfigAdmin, UUIDAdmin):
         'modified',
     ]
     inlines = [ConfigInline]
+    conditional_inlines = []
 
     org_position = 1 if not app_settings.HARDWARE_ID_ENABLED else 2
     list_display.insert(org_position, 'organization')
 
-    if app_settings.BACKEND_DEVICE_LIST:
+    if app_settings.CONFIG_BACKEND_FIELD_SHOWN:
         list_filter.insert(1, 'config__backend')
     if app_settings.HARDWARE_ID_ENABLED:
         list_display.insert(1, 'hardware_id')
@@ -420,7 +464,10 @@ class DeviceAdmin(MultitenantAdminMixin, BaseConfigAdmin, UUIDAdmin):
     list_select_related = ('config', 'organization')
 
     class Media(BaseConfigAdmin.Media):
-        js = BaseConfigAdmin.Media.js + ['{0}js/tabs.js'.format(prefix)]
+        js = BaseConfigAdmin.Media.js + [
+            f'{prefix}js/tabs.js',
+            f'{prefix}js/relevant_templates.js',
+        ]
 
     def get_fields(self, request, obj=None):
         """
@@ -446,71 +493,121 @@ class DeviceAdmin(MultitenantAdminMixin, BaseConfigAdmin, UUIDAdmin):
 
     def _get_preview_instance(self, request):
         c = super()._get_preview_instance(request)
-        c.device = self.model(
-            id=request.POST.get('id'),
-            name=request.POST.get('name'),
-            mac_address=request.POST.get('mac_address'),
-            key=request.POST.get('key'),
-        )
+        id_ = request.POST.get('id')
+        # instantiate new device if it's a new config
+        try:
+            c.device
+        except ObjectDoesNotExist:
+            c.device = self.model()
+        # fill attributes with up to date data
+        c.device.id = id_
+        c.device.name = request.POST.get('name')
+        c.device.mac_address = request.POST.get('mac_address')
+        c.device.key = request.POST.get('key')
         if 'hardware_id' in request.POST:
             c.device.hardware_id = request.POST.get('hardware_id')
         return c
 
-    def _get_default_template_urls(self):
-        """
-        returns URLs to get default templates
-        used in change_form.html template
-        """
-        organizations = Organization.active.all()
-        urls = {}
-        for org in organizations:
-            urls[str(org.pk)] = reverse('admin:get_default_templates', args=[org.pk])
-        return json.dumps(urls)
-
     def get_urls(self):
-        return [
-            url(
-                r'^config/get-default-templates/(?P<organization_id>[^/]+)/$',
-                get_default_templates,
-                name='get_default_templates',
+        urls = [
+            path(
+                'config/get-relevant-templates/<str:organization_id>/',
+                self.admin_site.admin_view(get_relevant_templates),
+                name='get_relevant_templates',
             ),
-            url(
-                r'^get-template-default-values/$',
-                get_template_default_values,
+            path(
+                'get-template-default-values/',
+                self.admin_site.admin_view(get_template_default_values),
                 name='get_template_default_values',
             ),
         ] + super().get_urls()
+        for inline in self.inlines + self.conditional_inlines:
+            try:
+                urls.extend(inline(self, self.admin_site).get_urls())
+            except AttributeError:
+                pass
+        return urls
 
     def get_extra_context(self, pk=None):
         ctx = super().get_extra_context(pk)
-        ctx.update({'default_template_urls': self._get_default_template_urls()})
+        ctx.update(
+            {
+                'relevant_template_url': reverse(
+                    'admin:get_relevant_templates', args=['org_id']
+                ),
+                'commands_api_endpoint': reverse(
+                    'connection_api:device_command_list',
+                    args=['00000000-0000-0000-0000-000000000000'],
+                ),
+            }
+        )
         return ctx
 
     def add_view(self, request, form_url='', extra_context=None):
         extra_context = self.get_extra_context()
         return super().add_view(request, form_url, extra_context)
 
+    def get_inlines(self, request, obj):
+        inlines = super().get_inlines(request, obj)
+        # this only makes sense in existing devices
+        if not obj:
+            return inlines
+        # add conditional_inlines if condition is met
+        inlines = list(inlines)  # copy
+        for inline in self.conditional_inlines:
+            inline_instance = inline(inline.model, admin.site)
+            if inline_instance._get_conditional_queryset(request, obj=obj):
+                inlines.append(inline)
+        return inlines
 
-if not app_settings.BACKEND_DEVICE_LIST:  # pragma: nocover
-    DeviceAdmin.list_display.remove('backend')
+    @classmethod
+    def add_reversion_following(cls, follow):
+        """
+        DeviceAdmin is used by other modules that register InlineModelAdmin
+        using monkey patching. The default implementation of reversion.register
+        ignores such inlines and does not update the "follow" field accordingly.
+        This method updates the "follow" fields of the Device model
+        by unregistering the Device model from reversion and re-registering it.
+        Only the" "follow" option is updated.
+        """
+        device_reversion_options = reversion.revisions._registered_models[
+            reversion.revisions._get_registration_key(Device)
+        ]
+        following = set(device_reversion_options.follow).union(set(follow))
+        reversion.unregister(Device)
+        reversion.register(
+            model=Device,
+            fields=device_reversion_options.fields,
+            follow=following,
+            format=device_reversion_options.format,
+            for_concrete_model=device_reversion_options.for_concrete_model,
+            ignore_duplicates=device_reversion_options.ignore_duplicates,
+            use_natural_foreign_keys=device_reversion_options.use_natural_foreign_keys,
+        )
 
 
 class CloneOrganizationForm(forms.Form):
-    organization = forms.ModelChoiceField(queryset=Organization.objects.none())
+    organization = forms.ModelChoiceField(
+        queryset=Organization.objects.none(),
+        required=False,
+        empty_label=_('Shared systemwide (no organization)'),
+    )
 
     def __init__(self, *args, **kwargs):
         queryset = kwargs.pop('queryset')
+        user = kwargs.pop('user')
         super().__init__(*args, **kwargs)
-        self.fields['organization'].queryset = queryset
+        org_field = self.fields.get('organization')
+        org_field.queryset = queryset
+        if not user.is_superuser:
+            org_field.empty_label = None
 
 
 class TemplateForm(BaseForm):
     class Meta(BaseForm.Meta):
         model = Template
         widgets = {'config': JsonSchemaWidget, 'default_values': FlatJsonWidget}
-        labels = {
-            'default_values': _('Configuration variables'),
-        }
+        labels = {'default_values': _('Configuration variables')}
         help_texts = {
             'default_values': _(
                 "If you want to use configuration variables in this template, "
@@ -528,6 +625,7 @@ class TemplateAdmin(MultitenantAdminMixin, BaseConfigAdmin, SystemDefinedVariabl
         'type',
         'backend',
         'default',
+        'required',
         'created',
         'modified',
     ]
@@ -536,6 +634,7 @@ class TemplateAdmin(MultitenantAdminMixin, BaseConfigAdmin, SystemDefinedVariabl
         'backend',
         'type',
         'default',
+        'required',
         'created',
     ]
     search_fields = ['name']
@@ -549,6 +648,7 @@ class TemplateAdmin(MultitenantAdminMixin, BaseConfigAdmin, SystemDefinedVariabl
         'auto_cert',
         'tags',
         'default',
+        'required',
         'system_context',
         'default_values',
         'config',
@@ -560,6 +660,17 @@ class TemplateAdmin(MultitenantAdminMixin, BaseConfigAdmin, SystemDefinedVariabl
     def clone_selected_templates(self, request, queryset):
         selectable_orgs = None
         user = request.user
+
+        def create_log_entry(user, clone):
+            ct = ContentType.objects.get(model='template')
+            LogEntry.objects.log_action(
+                user_id=user.id,
+                content_type_id=ct.pk,
+                object_id=clone.pk,
+                object_repr=clone.name,
+                action_flag=ADDITION,
+            )
+
         if user.is_superuser:
             all_orgs = Organization.objects.all()
             if all_orgs.count() > 1:
@@ -569,12 +680,14 @@ class TemplateAdmin(MultitenantAdminMixin, BaseConfigAdmin, SystemDefinedVariabl
                 pk__in=user.organizations_dict.keys()
             )
         if selectable_orgs:
-            if request.POST.get('organization'):
+            organization = request.POST.get('organization')
+            if organization or organization == '':
                 for template in queryset:
                     clone = template.clone(user)
-                    clone.organization = Organization.objects.get(
-                        pk=request.POST.get('organization')
-                    )
+                    clone.organization = None
+                    if organization:
+                        clone.organization = Organization.objects.get(pk=organization)
+                    create_log_entry(user, clone)
                     clone.save()
                 self.message_user(
                     request,
@@ -587,7 +700,7 @@ class TemplateAdmin(MultitenantAdminMixin, BaseConfigAdmin, SystemDefinedVariabl
                 'queryset': queryset,
                 'opts': self.model._meta,
                 'action_checkbox_name': helpers.ACTION_CHECKBOX_NAME,
-                'form': CloneOrganizationForm(queryset=selectable_orgs),
+                'form': CloneOrganizationForm(queryset=selectable_orgs, user=user),
                 'changelist_url': (
                     f'{request.resolver_match.app_name}:'
                     f'{request.resolver_match.url_name}'
@@ -599,11 +712,18 @@ class TemplateAdmin(MultitenantAdminMixin, BaseConfigAdmin, SystemDefinedVariabl
         else:
             for template in queryset:
                 clone = template.clone(user)
+                create_log_entry(user, clone)
                 clone.save()
 
     actions = ['clone_selected_templates']
 
 
+if not app_settings.CONFIG_BACKEND_FIELD_SHOWN:  # pragma: nocover
+    DeviceAdmin.list_display.remove('backend')
+    TemplateAdmin.list_display.remove('backend')
+    TemplateAdmin.list_filter.remove('backend')
+
+
 class VpnForm(forms.ModelForm):
     """
     Adds support for ``OPENWISP_CONTROLLER_VPN_BACKENDS``
@@ -630,16 +750,21 @@ class VpnAdmin(
     list_filter = [('organization', MultitenantOrgFilter), 'backend', 'created']
     search_fields = ['id', 'name', 'host', 'key']
     readonly_fields = ['id', 'uuid', 'system_context']
-    multitenant_shared_relations = ('ca', 'cert')
+    multitenant_shared_relations = ('ca', 'cert', 'subnet')
+    autocomplete_fields = ['ip', 'subnet']
     fields = [
+        'organization',
         'name',
         'host',
-        'organization',
         'uuid',
         'key',
+        'backend',
         'ca',
         'cert',
-        'backend',
+        'subnet',
+        'ip',
+        'webhook_endpoint',
+        'auth_token',
         'notes',
         'dh',
         'system_context',
@@ -652,9 +777,58 @@ class VpnAdmin(
         js = list(BaseConfigAdmin.Media.js) + [f'{prefix}js/vpn.js']
 
 
+class DeviceGroupForm(BaseForm):
+    class Meta(BaseForm.Meta):
+        model = DeviceGroup
+        widgets = {'meta_data': DeviceGroupJsonSchemaWidget}
+        labels = {'meta_data': _('Metadata')}
+        help_texts = {
+            'meta_data': _(
+                'Group meta data, use this field to store data which is related'
+                'to this group and can be retrieved via the REST API.'
+            )
+        }
+
+
+class DeviceGroupAdmin(MultitenantAdminMixin, BaseAdmin):
+    form = DeviceGroupForm
+    fields = [
+        'name',
+        'organization',
+        'description',
+        'meta_data',
+        'created',
+        'modified',
+    ]
+    search_fields = ['name', 'description', 'meta_data']
+    list_filter = [
+        ('organization', MultitenantOrgFilter),
+    ]
+
+    class Media:
+        css = {'all': (f'{prefix}css/admin.css',)}
+
+    def get_urls(self):
+        options = self.model._meta
+        url_prefix = f'{options.app_label}_{options.model_name}'
+        urls = super().get_urls()
+        urls += [
+            path(
+                f'{options.app_label}/{options.model_name}/ui/schema.json',
+                self.admin_site.admin_view(self.schema_view),
+                name=f'{url_prefix}_schema',
+            ),
+        ]
+        return urls
+
+    def schema_view(self, request):
+        return JsonResponse(app_settings.DEVICE_GROUP_SCHEMA)
+
+
 admin.site.register(Device, DeviceAdmin)
 admin.site.register(Template, TemplateAdmin)
 admin.site.register(Vpn, VpnAdmin)
+admin.site.register(DeviceGroup, DeviceGroupAdmin)
 
 
 if getattr(app_settings, 'REGISTRATION_ENABLED', True):
