@@ -1,12 +1,14 @@
 'use strict';
 (function ($) {
+    django._schemas = new Map();
+    django._jsonEditors = new Map();
     var inFullScreenMode = false,
         prevDefaultValues = {},
         defaultValuesUrl = window.location.origin + '/admin/config/device/get-template-default-values/',
     removeDefaultValues = function(contextValue, defaultValues) {
         // remove default values when template is removed.
         Object.keys(prevDefaultValues).forEach(function (key) {
-            if (!defaultValues.hasOwnProperty(key)) {
+            if (!defaultValues.hasOwnProperty(key) && contextValue.hasOwnProperty(key)) {
                 delete contextValue[key];
             }
         });
@@ -23,18 +25,20 @@
         return contextValue;
     },
     updateContext = function (isLoading, defaultValues={}) {
-        var contextField = $('#id_config-0-context');
-        if (contextField.length) {
-            var contextValue = JSON.parse(contextField.val());
+        var contextField = $('#id_config-0-context'),
+            systemContextField = $('#system_context');
+        if (contextField.length && systemContextField.length) {
+            var contextValue = JSON.parse(contextField.val()),
+                systemContextValue = JSON.parse(systemContextField.text());
             // add default values to contextValue
             Object.keys(defaultValues).forEach(function (key) {
-                if (!contextValue.hasOwnProperty(key)) {
+                if (!contextValue.hasOwnProperty(key) && !systemContextValue.hasOwnProperty(key)) {
                     contextValue[key] = defaultValues[key];
                 }
             });
 
-            if (isLoading) {
-                django._njc_initial_values['config-0-context'] = removeDefaultValues(
+            if (isLoading && django._owcInitialValues) {
+                django._owcInitialValues['config-0-context'] = removeDefaultValues(
                     contextValue,
                     defaultValues
                 );
@@ -66,7 +70,7 @@
         }
     },
     toggleFullScreen = function () {
-        var advanced = $('#advanced_editor');
+        var advanced = $('.advanced_editor:visible');
         if (!inFullScreenMode) {
             advanced.addClass('full-screen');
             $('html').addClass('editor-full');
@@ -84,8 +88,13 @@
     };
 
     var initAdvancedEditor = function (target, data, schema, disableSchema) {
-        var advanced = $("<div id='advanced_editor'></div>");
-        $(advanced).insertBefore($(target));
+        var advanced = $(target).prev('.advanced_editor');
+        if (advanced.length === 0){
+            advanced = $('<div class="advanced_editor"></div>');
+            $(advanced).insertBefore($(target));
+        } else {
+            advanced.empty();
+        }
         $(target).hide();
         // if disableSchema is true, do not validate againsts schema, default is false
         schema = disableSchema ? {} : schema;
@@ -102,7 +111,7 @@
             schema: schema
         };
 
-        var editor = new advancedJSONEditor(document.getElementById(advanced.attr('id')), options, data);
+        var editor = new advancedJSONEditor(advanced.get(0), options, data);
         editor.aceEditor.setOptions({
             fontSize: 14,
             showInvisibles: true
@@ -120,7 +129,7 @@
         // hide on esc button
         $('html').on('keydown', function (e) {
             if (inFullScreenMode && e.keyCode === 27) { // ESC
-                $('#advanced_editor').find('.jsoneditor-exit').click();
+                $('.advanced_editor:visible').find('.jsoneditor-exit').click();
             }
         });
         return editor;
@@ -151,6 +160,13 @@
         return error;
     };
 
+    var handleMaxLengthAttr = function() {
+        $('.jsoneditor input[maxlength]:not(.has-max-length)').map((i, field) => {
+            $(field).attr('data-maxlength', $(field).attr('maxLength'));
+        });
+        $('.jsoneditor input[maxlength]:not(.has-max-length)').addClass('has-max-length');
+    };
+
     var validateOnDefaultValuesChange = function (editor, advancedEditor) {
         window.isContextValid();
         if (inFullScreenMode) {
@@ -200,17 +216,30 @@
             disable_edit_json: true,
             startval: startval,
             keep_oneof_values: false,
-            show_errors: 'change',
+            show_errors: field.data('show-errors') ? field.data('show-errors'): 'change',
             // if no backend selected use empty schema
             schema: backend ? schemas[backend] : {}
         };
+        if (backend) {
+            options.schema = schemas[backend];
+        }
+        // single schema mode
+        else if (backend === false) {
+            options.schema = schemas;
+        }
+        // if no backend selected use empty schema
+        else {
+            options.schema = {};
+        }
         if (field.attr("data-options") !== undefined) {
             $.extend(options, JSON.parse(field.attr("data-options")));
         }
+
         editor = new JSONEditor(document.getElementById(id), options);
+        django._jsonEditors[id] = editor;
         // initialise advanced json editor here (disable schema validation in VPN admin)
         advancedEditor = initAdvancedEditor(field, value, options.schema, $('#vpn_form').length === 1);
-        $advancedEl = $('#advanced_editor');
+        $advancedEl = $(advancedEditor.container);
         getEditorValue = function () {
             return JSON.stringify(editor.getValue(), null, 4);
         };
@@ -228,7 +257,7 @@
         }
         // update raw value on change event
         editor.on('change', updateRaw);
-
+        editor.on('change', handleMaxLengthAttr);
         // update raw value before form submit
         form.submit(function (e) {
             // only submit form if the editor is clear of all validation errors
@@ -321,17 +350,29 @@
             }
         });
 
-        // fill device context field with default values of selected templates.
-        getDefaultValues(true);
+        // so that other files can use updateContext
+        window.updateContext = updateContext;
 
-        $('.sortedm2m-items').on('change', function() {
-            getDefaultValues();
+        $('.jsoneditor').on('input paste', '.has-max-length:visible', function(e) {
+            var field = $(e.target),
+                pasteValue = '';
+
+            if (e.originalEvent.type === 'paste') {
+                pasteValue = e.originalEvent.clipboardData.getData('text');
+            }
+
+            if (field.val().indexOf('{{') > -1 || pasteValue.indexOf('{{') > -1) {
+                field.removeAttr('maxlength');
+            } else {
+                field.attr('maxlength', field.data('maxlength'));
+            }
         });
     };
 
     var bindLoadUi = function () {
-        $.getJSON(django._jsonSchemaWidgetUrl, function (schemas) {
-            $('.jsoneditor-raw').each(function (i, el) {
+        $('.jsoneditor-raw:not([name*="__prefix__"])').each(function (i, el) {
+            $.getJSON($(el).data('schema-url'), function (schemas) {
+                django._schemas[$(el).data('schema-url')] = schemas;
                 var field = $(el),
                     schema = field.attr("data-schema"),
                     schemaSelector = field.attr("data-schema-selector");
@@ -341,26 +382,42 @@
                     if (schemaSelector === undefined) {
                         schemaSelector = '#id_backend, #id_config-0-backend';
                     }
-                    var selector = $(schemaSelector);
+                    var selector = $(schemaSelector),
+                        schemaKey = selector.val() || false;
                     // load first time
-                    loadUi(el, selector.val(), schemas, true);
+                    loadUi(el, schemaKey, schemas, true);
                     // reload when selector is changed
-                    selector.change(function () {
-                        loadUi(el, selector.val(), schemas);
-                    });
+                    if (selector.length) {
+                        selector.change(function () {
+                            loadUi(el, selector.val(), schemas);
+                        });
+                    }
                 }
-                $(document).trigger('jsonschema-schemaloaded');
+                $(`#${el.id}`).trigger('jsonschema-schemaloaded');
             });
         });
     };
 
     $(function () {
-        var add_config = $('#config-group.inline-group .add-row');
+        var addConfig = $('#config-group.inline-group .add-row');
         // if configuration is admin inline
         // load it when add button is clicked
-        add_config.click(bindLoadUi);
+        addConfig.click(bindLoadUi);
         // otherwise load immediately
         bindLoadUi();
+        // fill device context field with default values of selected templates.
+        // If unsaved_changes have already mapped values, then fetch defaultValues,
+        // otherwise wait for event to be triggered.
+        if (django._owcInitialValues !== undefined){
+            getDefaultValues(true);
+        } else {
+            $(document).one('owcInitialValuesLoaded', function () {
+                getDefaultValues(true);
+            });
+        }
+        $('.sortedm2m-items').on('change', function() {
+            getDefaultValues();
+        });
     });
 }(django.jQuery));
 
@@ -713,3 +770,144 @@ JSONEditor.defaults.themes.django = JSONEditor.AbstractTheme.extend({
         link.appendChild(image);
     }
 });
+
+// This method has been copied from jdorn/json-editor library to facilitate
+// overriding JSONEditor.defaults.editors.multiple.prototype.setValue
+JSONEditor.defaults.editors.multiple.prototype.$each = function (obj, callback) {
+    if (!obj || typeof obj !== "object") {
+        return;
+    }
+    var i;
+    if (Array.isArray(obj) || (typeof obj.length === 'number' && obj.length > 0 && (obj.length - 1) in obj)) {
+        for (i = 0; i < obj.length; i++) {
+            if (callback(i, obj[i]) === false) {
+                return;
+            }
+        }
+    } else {
+        if (Object.keys) {
+            var keys = Object.keys(obj);
+            for (i = 0; i < keys.length; i++) {
+                if (callback(keys[i], obj[keys[i]]) === false) {
+                    return;
+                }
+            }
+        } else {
+            for (i in obj) {
+                if (!obj.hasOwnProperty(i)) {
+                    continue;
+                }
+                if (callback(i, obj[i]) === false) {
+                    return;
+                }
+            }
+        }
+    }
+};
+
+// Override setValue method to allow using variables for fields with maxLength.
+// The code is copied from jdorn/json-editor library but contains customization
+// to remove maxLength attribute from schema of a field that has a value which
+// contains a variable: this customization is required for validation to pass
+// (the variable name could be longer than maxlength and may not fit).
+// Later, the maxLength attribute is added back to restore validator to it's original form.
+JSONEditor.defaults.editors.multiple.prototype.setValue = function (val, initial) {
+    // Determine type by getting the first one that validates
+    var self = this,
+        validatorModification = {};
+    this.$each(this.validators, function (i, validator) {
+        // Customization to modify validators starts here
+        if ((val) && typeof val === 'object') {
+            Object.entries(val).forEach(function (entry) {
+                if (typeof entry[1] === 'string' && entry[1].indexOf('{{') > -1) {
+                    if ((validator.schema.properties) && (validator.schema.properties[entry[0]])) {
+                        validatorModification[i] = {
+                            propertyName: entry[0],
+                            maxLength: validator.schema.properties[entry[0]].maxLength
+                        };
+                        delete validator.schema.properties[entry[0]].maxLength;
+                    }
+                }
+            });
+        }
+        // Customization to modify validators ends here
+        if (!validator.validate(val).length) {
+            self.type = i;
+            self.switcher.value = self.display_text[i];
+            return false;
+        }
+    });
+    this.switchEditor(this.type);
+
+    this.editors[this.type].setValue(val, initial);
+
+    // Customization to restore validators starts here
+    Object.entries(validatorModification).forEach(function (entry) {
+        self.validators[entry[0]].schema.properties[entry[1].propertyName].maxLength = entry[1].maxLength;
+    });
+    // Customization to restore validators ends here
+
+    this.refreshValue();
+    self.onChange();
+};
+
+// Overriding following methods on the JSONEditor is required for
+// working of select2 fields. Refer to https://git.io/J4Qcp for
+// more information.
+JSONEditor.defaults.editors.select.prototype.enable = function () {
+    if (!this.always_disabled) {
+        this.input.disabled = false;
+        if (this.select2) {
+            this.select2.disabled = false;
+        }
+    }
+    this.disabled = false;
+};
+
+JSONEditor.defaults.editors.select.prototype.disable = function () {
+    this.input.disabled = true;
+    if (this.select2) {
+        this.select2.disabled = true;
+    }
+    this.disabled = true;
+};
+
+JSONEditor.defaults.editors.multiselect.prototype.enable = function () {
+    if (!this.always_disabled) {
+        if (this.input) {
+            this.input.disabled = false;
+        } else if (this.inputs) {
+            for (var i in this.inputs) {
+                if (!this.inputs.hasOwnProperty(i)) {
+                    continue;
+                }
+                this.inputs[i].disabled = false;
+            }
+        }
+        // Modified code begins
+        if (this.select2) {
+            this.select2.disabled = false;
+        }
+        this.disabled = false;
+        // Modified code ends
+    }
+};
+
+JSONEditor.defaults.editors.multiselect.prototype.disable = function () {
+    if (this.input) {
+        this.input.disabled = true;
+    } else if (this.inputs) {
+        for (var i in this.inputs) {
+            if (!this.inputs.hasOwnProperty(i)) {
+                continue;
+            }
+            this.inputs[i].disabled = true;
+        }
+    }
+    // Modified code begins
+    if (this.select2) {
+        this.select2.disabled = true;
+        this._super();
+        // Modified code ends
+    }
+};
