@@ -4,6 +4,7 @@ from unittest.mock import patch
 from django.core.exceptions import ValidationError
 from django.db.transaction import atomic
 from django.test import TestCase
+from django.test.testcases import TransactionTestCase
 from netjsonconfig import OpenWrt
 from swapper import load_model
 
@@ -11,6 +12,7 @@ from openwisp_users.tests.utils import TestOrganizationMixin
 from openwisp_utils.tests import catch_signal
 
 from .. import settings as app_settings
+from ..base.config import logger as config_model_logger
 from ..signals import config_modified, config_status_changed
 from .utils import CreateConfigTemplateMixin, TestVpnX509Mixin
 
@@ -53,6 +55,58 @@ class TestConfig(
         c = Config(backend='netjsonconfig.OpenWrt', config=config)
         self.assertIsInstance(c.backend_instance, OpenWrt)
 
+    @patch.object(app_settings, 'DSA_DEFAULT_FALLBACK', False)
+    @patch.object(
+        app_settings,
+        'DSA_OS_MAPPING',
+        {
+            'netjsonconfig.OpenWrt': {
+                '>=21.02': [r'MyCustomFirmware 2.1(.*)'],
+                '<21.02': [r'MyCustomFirmware 2.0(.*)'],
+            }
+        },
+    )
+    def test_backend_openwrt_different_versions(self):
+        with self.subTest('DSA enabled OpenWrt firmware'):
+            c = Config(
+                backend='netjsonconfig.OpenWrt',
+                device=Device(name='test', os='OpenWrt 21.02.2 r16495-bf0c965af0'),
+            )
+            self.assertIsInstance(c.backend_instance, OpenWrt)
+            self.assertEqual(c.backend_instance.dsa, True)
+
+        with self.subTest('DSA disabed OpenWrt Firmware'):
+            c = Config(
+                backend='netjsonconfig.OpenWrt',
+                device=Device(name='test', os='OpenWrt 19.02.2 r16495-bf0c965af0'),
+            )
+            self.assertIsInstance(c.backend_instance, OpenWrt)
+            self.assertEqual(c.backend_instance.dsa, False)
+
+        with self.subTest('DSA enabled custom firmware'):
+            c = Config(
+                backend='netjsonconfig.OpenWrt',
+                device=Device(name='test', os='MyCustomFirmware 2.1.2'),
+            )
+            self.assertIsInstance(c.backend_instance, OpenWrt)
+            self.assertEqual(c.backend_instance.dsa, True)
+
+        with self.subTest('DSA disabled custom firmware'):
+            c = Config(
+                backend='netjsonconfig.OpenWrt',
+                device=Device(name='test', os='MyCustomFirmware 2.0.1'),
+            )
+            self.assertIsInstance(c.backend_instance, OpenWrt)
+            self.assertEqual(c.backend_instance.dsa, False)
+
+        with self.subTest('Device os field is empty'):
+            c = Config(
+                backend='netjsonconfig.OpenWrt',
+                device=Device(name='test', os=''),
+            )
+            self.assertIsInstance(c.backend_instance, OpenWrt)
+            self.assertEqual(c.backend_instance.dsa, False)
+
     def test_netjson_validation(self):
         config = {'interfaces': {'invalid': True}}
         c = Config(
@@ -127,6 +181,56 @@ class TestConfig(
         c = self._create_config(organization=self._get_org())
         self.assertEqual(len(c.checksum), 32)
 
+    def test_get_cached_checksum(self):
+        c = self._create_config(organization=self._get_org())
+
+        with self.subTest('check cache set'):
+            with patch('django.core.cache.cache.set') as mocked_set:
+                checksum = c.get_cached_checksum()
+                self.assertEqual(len(checksum), 32)
+                mocked_set.assert_called_once()
+
+        with self.subTest('check cache get'):
+            with patch(
+                'django.core.cache.cache.get', return_value=checksum
+            ) as mocked_get:
+                self.assertEqual(len(c.get_cached_checksum()), 32)
+                mocked_get.assert_called_once()
+
+        with self.subTest('ensure fresh checksum is calculated when cache is clear'):
+            with patch.object(config_model_logger, 'debug') as mocked_debug:
+                c.get_cached_checksum.invalidate(c)
+                self.assertEqual(len(c.get_cached_checksum()), 32)
+                mocked_debug.assert_called_once()
+
+        with self.subTest(
+            'ensure fresh checksum is NOT calculated when cache is present'
+        ):
+            with patch.object(config_model_logger, 'debug') as mocked_debug:
+                self.assertEqual(len(c.get_cached_checksum()), 32)
+                mocked_debug.assert_not_called()
+
+        with self.subTest('ensure cache invalidation works'):
+            with patch.object(config_model_logger, 'debug') as mocked_debug:
+                old_checksum = c.checksum
+                c.config['general']['timezone'] = 'Europe/Rome'
+                c.full_clean()
+                c.save()
+                del c.backend_instance
+                self.assertNotEqual(c.checksum, old_checksum)
+                self.assertEqual(c.get_cached_checksum(), c.checksum)
+                mocked_debug.assert_called_once()
+
+        with self.subTest('test cache invalidation when config templates are changed'):
+            with patch.object(config_model_logger, 'debug') as mocked_debug:
+                old_checksum = c.checksum
+                template = self._create_template()
+                c.templates.add(template)
+                del c.backend_instance
+                self.assertNotEqual(c.checksum, old_checksum)
+                self.assertEqual(c.get_cached_checksum(), c.checksum)
+                mocked_debug.assert_called_once()
+
     def test_backend_import_error(self):
         """
         see issue #5
@@ -271,8 +375,20 @@ class TestConfig(
         c.templates.clear()
         c.save()
         vpnclient = c.vpnclient_set.first()
-        self.assertIsNone(vpnclient)
-        self.assertEqual(c.vpnclient_set.count(), 0)
+        self.assertIsNotNone(vpnclient)
+        self.assertNotEqual(c.vpnclient_set.count(), 0)
+
+    def test_multiple_vpn_clients(self):
+        vpn1 = self._create_vpn(name='vpn1')
+        vpn2 = self._create_vpn(name='vpn2')
+        template1 = self._create_template(name='vpn1-template', type='vpn', vpn=vpn1)
+        template2 = self._create_template(name='vpn2-template', type='vpn', vpn=vpn2)
+        config = self._create_config(device=self._create_device())
+
+        config.templates.add(template1)
+        self.assertEqual(config.vpnclient_set.count(), 1)
+        config.templates.set((template1, template2))
+        self.assertEqual(config.vpnclient_set.count(), 2)
 
     def test_create_cert(self):
         vpn = self._create_vpn()
@@ -292,17 +408,17 @@ class TestConfig(
         c = Config.objects.get(device__name='test-create-cert')
         vpnclient = c.vpnclient_set.first()
         expected_cn = app_settings.COMMON_NAME_FORMAT.format(**c.device.__dict__)
-        self.assertEqual(vpnclient.cert.common_name, expected_cn)
+        self.assertIn(expected_cn, vpnclient.cert.common_name)
 
-    def test_automatically_created_cert_deleted_post_clear(self):
+    def test_automatically_created_cert_not_deleted_post_clear(self):
         self.test_create_cert()
         c = Config.objects.get(device__name='test-create-cert')
         vpnclient = c.vpnclient_set.first()
         cert = vpnclient.cert
         cert_model = cert.__class__
         c.templates.clear()
-        self.assertEqual(c.vpnclient_set.count(), 0)
-        self.assertEqual(cert_model.objects.filter(pk=cert.pk).count(), 0)
+        self.assertNotEqual(c.vpnclient_set.count(), 0)
+        self.assertNotEqual(cert_model.objects.filter(pk=cert.pk).count(), 0)
 
     def test_automatically_created_cert_deleted_post_remove(self):
         self.test_create_cert()
@@ -327,6 +443,36 @@ class TestConfig(
         self.assertIsNone(vpnclient.cert)
         self.assertEqual(c.vpnclient_set.count(), 1)
 
+    def test_cert_not_deleted_on_config_change(self):
+        vpn = self._create_vpn()
+        t = self._create_template(type='vpn', auto_cert=True, vpn=vpn)
+        c = self._create_config(device=self._create_device(name='test-device'))
+        c.templates.add(t)
+        c.save()
+        vpnclient = c.vpnclient_set.first()
+        cert = vpnclient.cert
+        cert_model = cert.__class__
+
+        with self.subTest(
+            "Ensure that the VpnClient and x509 Cert instance is created"
+        ):
+            self.assertIsNotNone(vpnclient)
+            self.assertTrue(vpnclient.auto_cert)
+            self.assertIsNotNone(vpnclient.cert)
+
+        c.templates.clear()
+        with self.subTest("Ensure that VpnClient and Cert instance are not deleted"):
+            self.assertIsNotNone(c.vpnclient_set.first())
+            self.assertNotEqual(c.vpnclient_set.count(), 0)
+            self.assertNotEqual(cert_model.objects.filter(pk=cert.pk).count(), 0)
+
+        # add the template again
+        c.templates.add(t)
+        c.save()
+        with self.subTest("Ensure no additional VpnClients are created"):
+            self.assertEqual(c.vpnclient_set.count(), 1)
+            self.assertEqual(c.vpnclient_set.first(), vpnclient)
+
     def _get_vpn_context(self):
         self.test_create_cert()
         c = Config.objects.get(device__name='test-create-cert')
@@ -533,13 +679,9 @@ class TestConfig(
         with catch_signal(config_status_changed) as handler:
             c.config = {'general': {'description': 'test'}}
             c.full_clean()
-            self.assertEqual(c.status, 'modified')
-            handler.assert_not_called()
-
-        with catch_signal(config_status_changed) as handler:
             c.save()
             handler.assert_called_once_with(
-                sender=Config, signal=config_status_changed, instance=c,
+                sender=Config, signal=config_status_changed, instance=c
             )
             self.assertEqual(c.status, 'modified')
 
@@ -560,15 +702,6 @@ class TestConfig(
         with catch_signal(config_modified) as handler:
             c.config = {'general': {'description': 'test'}}
             c.full_clean()
-            handler.assert_not_called()
-            self.assertEqual(c.status, 'modified')
-
-        with catch_signal(config_modified) as handler:
-            c.config = {'general': {'description': 'hey'}}
-            c.full_clean()
-            handler.assert_not_called()
-
-        with catch_signal(config_modified) as handler:
             c.save()
             handler.assert_called_once_with(
                 sender=Config,
@@ -576,20 +709,73 @@ class TestConfig(
                 instance=c,
                 device=c.device,
                 config=c,
+                previous_status='applied',
+                action='config_changed',
             )
             self.assertEqual(c.status, 'modified')
 
         with catch_signal(config_modified) as handler:
             c.config = {'general': {'description': 'changed again'}}
             c.full_clean()
+            # repeated on purpose
             c.full_clean()
             c.save()
-            handler.assert_called_once()
+            handler.assert_called_once_with(
+                sender=Config,
+                signal=config_modified,
+                instance=c,
+                device=c.device,
+                config=c,
+                previous_status='modified',
+                action='config_changed',
+            )
             self.assertEqual(c.status, 'modified')
 
+    def test_check_changes_query(self):
+        config = self._create_config(organization=self._get_org())
+        with self.assertNumQueries(1):
+            config._check_changes()
+
     def test_config_get_system_context(self):
         config = self._create_config(
             organization=self._get_org(), context={'test': 'value'}
         )
         system_context = config.get_system_context()
         self.assertNotIn('test', system_context.keys())
+
+    def test_initial_status(self):
+        config = self._create_config(
+            organization=self._get_org(), context={'test': 'value'}
+        )
+        self.assertEqual(config._initial_status, config.status)
+        config.status = 'modified'
+        config.save()
+        self.assertEqual(config._initial_status, 'modified')
+
+
+class TestTransactionConfig(
+    CreateConfigTemplateMixin,
+    TestOrganizationMixin,
+    TestVpnX509Mixin,
+    TransactionTestCase,
+):
+    def test_certificate_renew_invalidates_checksum_cache(self):
+        config = self._create_config(organization=self._get_org())
+        vpn_template = self._create_template(
+            name='vpn1-template', type='vpn', vpn=self._create_vpn(), config={}
+        )
+        config.templates.add(vpn_template)
+        config.refresh_from_db()
+        with patch('django.core.cache.cache.delete') as mocked_delete:
+            # Comparing checksum values after deleting backend instance
+            # makes the test bogus. Hence assertion for cache.delete is required
+            old_checksum = config.checksum
+            vpnclient_cert = config.vpnclient_set.first().cert
+            vpnclient_cert.renew()
+            # An additional call from cache invalidation of
+            # DeviceGroupCommonName View
+            self.assertEqual(mocked_delete.call_count, 3)
+            del config.backend_instance
+            self.assertNotEqual(config.get_cached_checksum(), old_checksum)
+            config.refresh_from_db()
+            self.assertEqual(config.status, 'modified')
