@@ -6,19 +6,28 @@ from django.test import TestCase
 from swapper import load_model
 
 from openwisp_users.tests.utils import TestOrganizationMixin
+from openwisp_utils.tests import AssertNumQueriesSubTestMixin, catch_signal
 
 from .. import settings as app_settings
+from ..signals import device_group_changed, device_name_changed, management_ip_changed
 from ..validators import device_name_validator, mac_address_validator
-from .utils import CreateConfigTemplateMixin
+from .utils import CreateConfigTemplateMixin, CreateDeviceGroupMixin
 
 TEST_ORG_SHARED_SECRET = 'functional_testing_secret'
 
 Config = load_model('config', 'Config')
 Device = load_model('config', 'Device')
+DeviceGroup = load_model('config', 'DeviceGroup')
 _original_context = app_settings.CONTEXT.copy()
 
 
-class TestDevice(CreateConfigTemplateMixin, TestOrganizationMixin, TestCase):
+class TestDevice(
+    CreateConfigTemplateMixin,
+    TestOrganizationMixin,
+    AssertNumQueriesSubTestMixin,
+    CreateDeviceGroupMixin,
+    TestCase,
+):
     """
     tests for Device model
     """
@@ -35,7 +44,7 @@ class TestDevice(CreateConfigTemplateMixin, TestOrganizationMixin, TestCase):
         self.assertEqual(str(d), '123')
 
     def test_mac_address_validator(self):
-        d = Device(name='test', key=self.TEST_KEY)
+        d = Device(name='test', key=self.TEST_KEY, organization=self._get_org())
         bad_mac_addresses_list = [
             '{0}:BB:CC'.format(self.TEST_MAC_ADDRESS),
             'AA:BB:CC:11:22033',
@@ -63,7 +72,6 @@ class TestDevice(CreateConfigTemplateMixin, TestOrganizationMixin, TestCase):
         self.assertEqual(c.status, 'modified')
 
     def test_key_validator(self):
-
         d = Device(
             organization=self._get_org(),
             name='test',
@@ -106,11 +114,11 @@ class TestDevice(CreateConfigTemplateMixin, TestOrganizationMixin, TestCase):
     def test_get_default_templates(self):
         d = self._create_device()
         self.assertEqual(
-            d.get_default_templates().count(), Config().get_default_templates().count(),
+            d.get_default_templates().count(), Config().get_default_templates().count()
         )
         self._create_config(device=d)
         self.assertEqual(
-            d.get_default_templates().count(), Config().get_default_templates().count(),
+            d.get_default_templates().count(), Config().get_default_templates().count()
         )
 
     def test_bad_hostnames(self):
@@ -225,6 +233,41 @@ class TestDevice(CreateConfigTemplateMixin, TestOrganizationMixin, TestCase):
         with self.assertRaises(ValidationError):
             self._create_device(**kwargs)
 
+    def test_device_group_validation(self):
+        org1 = self._get_org()
+        org2 = self._create_org(name='org2')
+        kwargs = {
+            'name': 'test.device.name',
+            'organization': org1,
+            'mac_address': '0a-1b-3c-4d-5e-6f',
+            'group': self._create_device_group(organization=org2),
+        }
+        device = Device(**kwargs)
+        with self.assertRaises(ValidationError) as context_manager:
+            device.full_clean()
+        expected_error_dict = {
+            'group': [
+                'Please ensure that the organization of this Device and the '
+                'organization of the related Device Group match.'
+            ]
+        }
+        self.assertEqual(context_manager.exception.message_dict, expected_error_dict)
+
+    @mock.patch('openwisp_controller.config.settings.DEVICE_NAME_UNIQUE', False)
+    def test_device_name_organization_not_unique(self):
+        org = self._get_org()
+        self._create_device(organization=org, name='test.device.name')
+        kwargs = {
+            'name': 'test.device.name',
+            'organization': org,
+            'mac_address': '0a-1b-3c-4d-5e-6f',
+        }
+        try:
+            d = Device(**kwargs)
+            d.full_clean()
+        except ValidationError:
+            self.fail('Validation error was raised')
+
     def test_device_macaddress_organization_unique_together(self):
         org = self._get_org()
         self._create_device(
@@ -241,9 +284,7 @@ class TestDevice(CreateConfigTemplateMixin, TestOrganizationMixin, TestCase):
     def test_device_hardwareid_unique_together(self):
         org = self._get_org()
         self._create_device(
-            organization=org,
-            hardware_id='098H52ST479QE053V2',
-            name='test.device3.name',
+            organization=org, hardware_id='098H52ST479QE053V2', name='test.device3.name'
         )
         kwargs = {
             'organization': org,
@@ -274,3 +315,96 @@ class TestDevice(CreateConfigTemplateMixin, TestOrganizationMixin, TestCase):
         d.refresh_from_db()
         system_context = d.get_system_context()
         self.assertNotIn('test', system_context.keys())
+
+    def test_management_ip_changed_not_emitted_on_creation(self):
+        with catch_signal(management_ip_changed) as handler:
+            self._create_device(organization=self._get_org())
+        handler.assert_not_called()
+
+    def test_management_ip_changed_emitted(self):
+        device = self._create_device(organization=self._get_org())
+
+        with catch_signal(management_ip_changed) as handler:
+            device.management_ip = '0.0.0.0'
+            device.save()
+        handler.assert_called_once_with(
+            management_ip='0.0.0.0',
+            old_management_ip=None,
+            sender=Device,
+            signal=management_ip_changed,
+            instance=device,
+        )
+
+    def test_name_unique_validation(self):
+        org = self._get_org()
+        device = self._create_device(name='test', organization=org)
+        self.assertEqual(device.name, 'test')
+        org2 = self._create_org(name='test org2', slug='test-org2')
+        device2 = self._create_device(
+            name='test', organization=org2, mac_address='0a:1b:3c:4d:5e:6f'
+        )
+        self.assertEqual(device2.name, 'test')
+        with self.assertRaises(ValidationError) as context_manager:
+            self._create_device(
+                name='test', organization=org2, mac_address='0a:1b:3c:4d:5e:1a'
+            )
+        message_dict = context_manager.exception.message_dict
+        self.assertIn('__all__', message_dict)
+        self.assertEqual(
+            ['Device with this Name and Organization already exists.'],
+            message_dict['__all__'],
+        )
+
+    def test_device_name_changed_emitted(self):
+        org = self._get_org()
+        device = self._create_device(name='test', organization=org)
+
+        with catch_signal(device_name_changed) as handler:
+            device.name = 'newtestdevice'
+            device.save()
+            self.assertEqual(device.name, 'newtestdevice')
+            handler.assert_called_once_with(
+                sender=Device, instance=device, signal=device_name_changed
+            )
+
+    def test_device_name_changed_not_emitted_on_creation(self):
+        with catch_signal(device_name_changed) as handler:
+            self._create_device(organization=self._get_org())
+        handler.assert_not_called()
+
+    def test_device_group_changed_emitted(self):
+        org = self._get_org()
+        device = self._create_device(name='test', organization=org)
+        device_group = self._create_device_group()
+
+        with catch_signal(device_group_changed) as handler:
+            device.group = device_group
+            device.save()
+            handler.assert_called_once_with(
+                signal=device_group_changed,
+                sender=Device,
+                instance=device,
+                old_group_id=None,
+                group_id=device_group.id,
+            )
+
+    def test_device_group_changed_not_emitted_on_creation(self):
+        with catch_signal(device_group_changed) as handler:
+            self._create_device(organization=self._get_org())
+        handler.assert_not_called()
+
+    def test_device_field_changed_checks(self):
+        self._create_device()
+        device_group = self._create_device_group()
+        with self.subTest('Deferred fields remained deferred'):
+            device = Device.objects.only('id', 'created').first()
+            device._check_changed_fields()
+
+        with self.subTest('Deferred fields becomes non-deferred'):
+            device.name = 'new-name'
+            device.management_ip = '10.0.0.1'
+            device.group_id = device_group.id
+            # Another query is generated due to "config.set_status_modified"
+            # on name change
+            with self.assertNumQueries(3):
+                device._check_changed_fields()
