@@ -3,6 +3,4 @@ from django.urls import path
 from .views import schema
 
 app_name = 'openwisp_controller'
-urlpatterns = [
-    path('config/schema.json', schema, name='schema'),
-]
+urlpatterns = [path('config/schema.json', schema, name='schema')]
