@@ -4,6 +4,7 @@ from io import BytesIO, StringIO
 
 import paramiko
 from django.utils.functional import cached_property
+from django.utils.translation import gettext_lazy as _
 from jsonschema import validate
 from jsonschema.exceptions import ValidationError as SchemaError
 from scp import SCPClient
@@ -63,6 +64,8 @@ class Ssh(object):
     def validate(cls, params):
         validate(params, cls.schema)
         cls.custom_validation(params)
+        # trigger SSH key algorithm check
+        cls(params, ['127.0.0.1']).params
 
     @classmethod
     def custom_validation(cls, params):
@@ -73,10 +76,29 @@ class Ssh(object):
     def params(self):
         params = self._params.copy()
         if 'key' in params:
-            key_fileobj = StringIO(params.pop('key'))
-            params['pkey'] = paramiko.RSAKey.from_private_key(key_fileobj)
+            params['pkey'] = self._get_ssh_key(params.pop('key'))
         return params
 
+    def _get_ssh_key(self, key):
+        key_fileobj = StringIO(key)
+        key_algorithms = [
+            paramiko.RSAKey,
+            paramiko.Ed25519Key,
+        ]
+        for key_algo in key_algorithms:
+            try:
+                return getattr(key_algo, 'from_private_key')(key_fileobj)
+            except (paramiko.ssh_exception.SSHException, ValueError):
+                key_fileobj.seek(0)
+                continue
+        else:
+            raise SchemaError(
+                _(
+                    'Unrecognized or unsupported SSH key algorithm, '
+                    'only RSA and ED25519 are currently supported.'
+                )
+            )
+
     def connect(self):
         success = False
         exception = None
@@ -118,7 +140,7 @@ class Ssh(object):
         - aborts on exceptions
         - raises socket.timeout exceptions
         """
-        print('$:> {0}'.format(command))
+        logger.info('Executing command: {0}'.format(command))
         # execute commmand
         try:
             stdin, stdout, stderr = self.shell.exec_command(command, timeout=timeout)
@@ -133,19 +155,25 @@ class Ssh(object):
         # store command exit status
         exit_status = stdout.channel.recv_exit_status()
         # log standard output
-        output = stdout.read().decode('utf8').strip()
+        # try to decode to UTF-8, ignoring unconvertible characters
+        # https://docs.python.org/3/howto/unicode.html#the-string-type
+        output = stdout.read().decode('utf-8', 'ignore')
         if output:
-            print(output)
+            logger.info(output)
         # log standard error
-        error = stderr.read().decode('utf8').strip()
+        error = stderr.read().decode('utf-8', 'ignore')
         if error:
-            print(error)
+            if not output.endswith('\n'):
+                output += '\n'
+            output += error
         # abort the operation if any of the command
         # returned with a non-zero exit status
         if exit_status not in exit_codes and raise_unexpected_exit:
-            print('# Previous command failed, aborting...')
+            log_message = 'Unexpected exit code: {0}'.format(exit_status)
+            logger.info(log_message)
             message = error if error else output
-            raise CommandFailedException(message)
+            # if message is empty, use log_message
+            raise CommandFailedException(message or log_message)
         return output, exit_status
 
     def update_config(self):  # pragma: no cover
