@@ -1,6 +1,7 @@
+import swapper
 from django_loci.channels.base import BaseLocationBroadcast
 
-from ..models import Location
+Location = swapper.load_model('geo', 'Location')
 
 
 class LocationBroadcast(BaseLocationBroadcast):
@@ -12,7 +13,7 @@ class LocationBroadcast(BaseLocationBroadcast):
         if (
             result
             and not user.is_superuser
-            and ((location.organization.pk,) not in user.organizations_pk)
+            and not user.is_manager(location.organization)
         ):
             return False
         return result
