@@ -1,7 +1,7 @@
 from channels.auth import AuthMiddlewareStack
 from channels.routing import ProtocolTypeRouter, URLRouter
 from channels.security.websocket import AllowedHostsOriginValidator
-from django.conf.urls import url
+from django.urls import path
 from django_loci.channels.base import location_broadcast_path
 from openwisp_notifications.websockets.routing import (
     get_routes as get_notification_routes,
@@ -11,7 +11,11 @@ from .consumers import LocationBroadcast
 
 
 def get_routes():
-    return [url(location_broadcast_path, LocationBroadcast, name='LocationChannel')]
+    return [
+        path(
+            location_broadcast_path, LocationBroadcast.as_asgi(), name='LocationChannel'
+        )
+    ]
 
 
 # Kept for backward compatibility
@@ -20,7 +24,7 @@ geo_routes = get_routes()
 channel_routing = ProtocolTypeRouter(
     {
         'websocket': AllowedHostsOriginValidator(
-            AuthMiddlewareStack(URLRouter(get_notification_routes() + geo_routes)),
+            AuthMiddlewareStack(URLRouter(get_notification_routes() + geo_routes))
         )
     }
 )
