@@ -1,14 +1,21 @@
 from django.contrib import admin
 from django_x509.base.admin import AbstractCaAdmin, AbstractCertAdmin
 from reversion.admin import VersionAdmin
+from swapper import load_model
 
 from openwisp_users.multitenancy import MultitenantOrgFilter
 
 from ..admin import MultitenantAdminMixin
-from .models import Ca, Cert
+from .base import PkiReversionTemplatesMixin
+
+Ca = load_model('django_x509', 'Ca')
+Cert = load_model('django_x509', 'Cert')
 
 
-class CaAdmin(MultitenantAdminMixin, VersionAdmin, AbstractCaAdmin):
+@admin.register(Ca)
+class CaAdmin(
+    MultitenantAdminMixin, PkiReversionTemplatesMixin, AbstractCaAdmin, VersionAdmin
+):
     history_latest_first = True
 
 
@@ -18,7 +25,10 @@ CaAdmin.list_display.insert(1, 'organization')
 CaAdmin.Media.js += ('admin/pki/js/show-org-field.js',)
 
 
-class CertAdmin(MultitenantAdminMixin, VersionAdmin, AbstractCertAdmin):
+@admin.register(Cert)
+class CertAdmin(
+    MultitenantAdminMixin, PkiReversionTemplatesMixin, AbstractCertAdmin, VersionAdmin
+):
     multitenant_shared_relations = ('ca',)
     history_latest_first = True
 
@@ -28,7 +38,3 @@ CertAdmin.list_filter.insert(0, ('organization', MultitenantOrgFilter))
 CertAdmin.list_filter.remove('ca')
 CertAdmin.list_display.insert(1, 'organization')
 CertAdmin.Media.js += ('admin/pki/js/show-org-field.js',)
-
-
-admin.site.register(Ca, CaAdmin)
-admin.site.register(Cert, CertAdmin)
