@@ -1,13 +1,13 @@
 from django.contrib.auth.models import Permission
 
-from ...migrations import create_default_permissions
+from ...migrations import create_default_permissions, get_swapped_model
 
 
 def assign_permissions_to_groups(apps, schema_editor):
     create_default_permissions(apps, schema_editor)
     operators_read_only_admins_manage = ['ca', 'cert']
     manage_operations = ['add', 'change', 'delete']
-    Group = apps.get_model('openwisp_users', 'Group')
+    Group = get_swapped_model(apps, 'openwisp_users', 'Group')
 
     try:
         admin = Group.objects.get(name='Administrator')
